@@ -2,10 +2,29 @@
 
 # Notification Center code borrowed from https://github.com/maranas/pyNotificationCenter/blob/master/pyNotificationCenter.py
 
+VERSION =   '1.1.0'
+
 import os, sys, json, re, hashlib, argparse, urllib, time
 
+# disable import warnings
+import warnings
+warnings.filterwarnings('ignore')
+
+DEFAULT_COLOR   =   '\033[0;0m'
+ERROR_COLOR     =   '\033[01;31m'
+ALERT_COLOR     =   '\033[01;33m'
+
+def alert(text, error_code = None, color = None):
+    if error_code is not None:
+        sys.stderr.write('%s%s%s\n' % (color or ERROR_COLOR, text, DEFAULT_COLOR))
+        sys.stderr.flush()
+        sys.exit(error_code)
+    else:
+        sys.stdout.write('%s%s%s\n' % (color or DEFAULT_COLOR, text, DEFAULT_COLOR))
+        sys.stdout.flush()
+
 # check for updates
-GIST_URL        =   'https://api.github.com/gists/5317321'
+GIST_URL        =   'https://api.github.com/repos/dryan/d3ploy/contents/d3ploy.py'
 CHECK_FILE      =   os.path.expanduser('~/.d3ploy-update-check')
 if not os.path.exists(CHECK_FILE):
     try:
@@ -20,24 +39,21 @@ now =   int(time.time())
 if now - last_checked > 86400:
     # it has been a day since the last update check
     try:
-        gist_hash       =   hashlib.md5(json.load(urllib.urlopen(GIST_URL)).get('files').get('d3ploy.py').get('content')).hexdigest()
+        gist_hash       =   hashlib.md5(json.load(urllib.urlopen(GIST_URL)).get('content')).hexdigest()
         script_hash     =   hashlib.md5(open(__file__, 'r').read()).hexdigest()
         check_file      =   open(CHECK_FILE, 'w')
         check_file.write('%d' % now)
         check_file.flush()
         check_file.close()
         if not gist_hash == script_hash:
-            sys.stdout.write('There has been an update for d3ploy.\nPlease see https://gist.github.com/dryan/5317321 or run `pip install --upgrade d3ploy`.\n')
-            sys.stdout.flush()
+            alert('There has been an update for d3ploy.\nPlease see https://github.com/dryan/d3ploy or run `pip install --upgrade d3ploy`.', color = ALERT_COLOR)
     except:
         pass
 
 try:
     import boto
 except ImportError:
-    sys.stdout.write("Please install boto. `pip install boto`\n")
-    sys.stdout.flush()
-    sys.exit(os.EX_UNAVAILABLE)
+    alert("Please install boto. `pip install boto`", os.EX_UNAVAILABLE)
     
 try:
     import Foundation, objc
@@ -53,8 +69,7 @@ if notifications:
         notifications   =   False
         
 def notify(env, text):
-    sys.stdout.write(text + '\n')
-    sys.stdout.flush()
+    alert(text)
     if notifications:
         notification    =   NSUserNotification.alloc().init()
         notification.setTitle_('d3ploy')
@@ -70,9 +85,7 @@ def notify(env, text):
 try:
     config      =   open('deploy.json', 'r')
 except IOError:
-    sys.stdout.write("deploy.json file is missing. See https://gist.github.com/dryan/5317321 for more information.\n")
-    sys.stdout.flush()
-    sys.exit(os.EX_NOINPUT)
+    alert("deploy.json file is missing. See https://gist.github.com/dryan/5317321 for more information.", os.EX_NOINPUT)
 
 config          =   json.load(config)
 
@@ -86,12 +99,20 @@ parser.add_argument('-a', '--access-key', help = "AWS Access Key ID", type = str
 parser.add_argument('-s', '--access-secret', help = "AWS Access Key Secret", type = str)
 parser.add_argument('-f', '--force', help = "Upload all files whether they are currently up to date on S3 or not", action = "store_true", default = False)
 parser.add_argument('--no-delete', help = "Don't remove orphaned files from S3", action = "store_true", default = False)
+parser.add_argument('--delete', help = "Remove orphaned files from S3", action = "store_true", default = False)
 parser.add_argument('--all', help = "Upload to all environments", action = "store_true", default = False)
 parser.add_argument('-n', '--dry-run', help = "Show which files would be updated without uploading to S3", action = "store_true", default = False)
 parser.add_argument('--acl', help = "The ACL to apply to uploaded files.", type = str, default = "public-read", choices = valid_acls)
+parser.add_argument('-v', '--version', help = "Print the script version and exit", action = "store_true", default = False)
 
 args            =   parser.parse_args()
 
+if args.version:
+    alert('d3ploy %s' % VERSION, os.EX_OK, DEFAULT_COLOR)
+
+if args.no_delete:
+    alert('--no-delete has been deprecated. Orphaned files will only be deleted if --delete is specified.')
+
 AWS_KEY         =   args.access_key
 AWS_SECRET      =   args.access_secret
 
@@ -109,23 +130,18 @@ if AWS_SECRET is None:
     AWS_SECRET  =   os.environ.get('AWS_SECRET_ACCESS_KEY')
     
 def upload_files(env, config):
-    sys.stdout.write('Using settings for "%s" environment\n' % env)
-    sys.stdout.flush()
+    alert('Using settings for "%s" environment' % env)
     
     bucket              =   config.get('bucket')
     if not bucket:
-        sys.stdout.write('A bucket to upload to was not specified for "%s" environment\n' % args.environment)
-        sys.stdout.flush()
-        sys.exit(os.EX_NOINPUT)
+        alert('A bucket to upload to was not specified for "%s" environment' % args.environment, os.EX_NOINPUT)
 
     KEY         =   config.get('aws_key', AWS_KEY)
 
     SECRET      =   config.get('aws_secret', AWS_SECRET)
     
     if KEY is None or SECRET is None:
-        sys.stdout.write("AWS credentials were not found. See https://gist.github.com/dryan/5317321 for more information.\n")
-        sys.stdout.flush()
-        sys.exit(os.EX_NOINPUT)
+        alert("AWS credentials were not found. See https://gist.github.com/dryan/5317321 for more information.", os.EX_NOINPUT)
     
     s3connection        =   boto.connect_s3(KEY, SECRET)
 
@@ -133,9 +149,7 @@ def upload_files(env, config):
     try:
         s3bucket        =   s3connection.get_bucket(bucket)
     except boto.exception.S3ResponseError:
-        sys.stdout.write('Bucket "%s" could not be retrieved with the specified credentials\n' % bucket)
-        sys.stdout.flush()
-        sys.exit(os.EX_NOINPUT)
+        alert('Bucket "%s" could not be retrieved with the specified credentials' % bucket, os.EX_NOINPUT)
 
     # get the rest of the options
     local_path          =   config.get('local_path', '.')
@@ -181,8 +195,7 @@ def upload_files(env, config):
                 break
             md5.update(data)
         if s3key is None or args.force or not s3key.etag.strip('"') == md5.hexdigest():
-            sys.stdout.write('Copying %s to %s%s\n' % (filename, bucket, keyname))
-            sys.stdout.flush()
+            alert('Copying %s to %s%s' % (filename, bucket, keyname))
             updated     +=  1
             if args.dry_run:
                 continue
@@ -191,31 +204,28 @@ def upload_files(env, config):
             s3key.set_contents_from_filename(filename)
             s3key.set_acl(args.acl)
     
-    if not args.no_delete:
+    if args.delete:
         for key in s3bucket.list(prefix = bucket_path.lstrip('/')):
             if not key.name in keynames:
-                sys.stdout.write('Deleting %s/%s\n' % (bucket, key.name.lstrip('/')))
-                sys.stdout.flush()
+                alert('Deleting %s/%s' % (bucket, key.name.lstrip('/')))
                 deleted     +=  1
                 if args.dry_run:
                     continue
                 key.delete()
         
     verb    =   "would be" if args.dry_run else "were"
-    notify(args.environment, "%d files %s updated and %d files %s removed\n" % (updated, verb, deleted, verb))
-    sys.stdout.write("")
-    sys.stdout.flush()
+    notify(args.environment, "%d files %s updated" % (updated, verb))
+    if args.delete:
+        notify(args.environment, "%d files %s removed" % (deleted, verb))
+    alert("")
 
 if not args.environment in config:
-    sys.stdout.write('The "%s" environment was not found in deploy.json\n' % args.environment)
-    sys.stdout.flush()
-    sys.exit(os.EX_NOINPUT)
+    alert('The "%s" environment was not found in deploy.json' % args.environment, os.EX_NOINPUT)
 
 def main():
     if args.all:
         for environ in config:
-            sys.stdout.write("Uploading environment %d of %d\n" % (config.keys().index(environ) + 1, len(config.keys())))
-            sys.stdout.flush()
+            alert("Uploading environment %d of %d" % (config.keys().index(environ) + 1, len(config.keys())))
             upload_files(environ, config[environ])
     else:
         upload_files(args.environment, config[args.environment])
